# FILE: bug75.tcl
#
#	Patches for Tcl 7.5 / Tk 4.1
#
# Last modified on: $Date: 1999/05/12 14:15:01 $
# Last modified by: $Author: dgp $
#
# This file contains Tcl code which when sourced in a Tcl 7.5 interpreter
# will patch bugs in Tcl 7.5 fixed in later versions of Tcl.  It is not
# meant to be complete.  Bugs are patched here only as we discover the need
# to do so in writing other OOMMF software.

Oc_Log Log "Patching Tcl 7.5 bugs..." status

if {[string match unix $tcl_platform(platform)]} {

    # The subcommands copy, delete, rename, and mkdir were added to
    # the Tcl command 'file' in Tcl version 7.6.  The following command
    # approximates them on Unix platforms.  It may not agree with
    # the Tcl 7.6+ command 'file' in all of its functionality (notably
    # the way it reports errors).  Further refinements should be made as
    # needed.
    rename file Tcl7.5_file
    proc file {option args} {
        switch -exact -- $option {
            c    -
            co   -
            cop  -
            copy {
                # Translate -force into -f
                if {[string match -force [lindex $args 0]]} {
                    set args [lreplace $args 0 0 -f]
                }
                uplevel exec cp $args
            }
            de     -
            del    -
            dele   -
            delet  -
            delete {
                if {[string match -force [lindex $args 0]]} {
                    set args [lreplace $args 0 0 -f]
                }
                catch {uplevel exec rm $args}
            }
            mk    -
            mkd   -
            mkdi  -
            mkdir {
                uplevel exec mkdir $args
            }
            ren    -
            rena   -
            renam  -
            rename {
                if {[string match -force [lindex $args 0]]} {
                    set args [lreplace $args 0 0 -f]
                }
                uplevel exec mv $args
            }
            default {
                uplevel [list Tcl7.5_file $option] $args
            }
        }
    }

    # Use the auto_execok implementation from Tcl 7.6 and later.
    # Only need the Unix version.
    proc auto_execok name {
	global auto_execs env

	if {[info exists auto_execs($name)]} {
            return $auto_execs($name)
        }
        set auto_execs($name) ""
        if {[llength [file split $name]] != 1} {
            if {[file executable $name] && ![file isdirectory $name]} {
                set auto_execs($name) [list $name]
            }
            return $auto_execs($name)
        }
        foreach dir [split $env(PATH) :] {
            if {$dir == ""} {
                set dir .
            }
            set file [file join $dir $name]
            if {[file executable $file] && ![file isdirectory $file]} {
                set auto_execs($name) [list $file]
                return $auto_execs($name)
            }
        }
        return ""
    }

    # The subcommands remove was added to the Tk command 'grid' in Tk
    # version 4.1.  The option -pad was also added to the subcommands
    # columnconfigure and rowconfigure.  The syntax of the bbox
    # subcommand also changed in a backward compatible way.
    # Also, [grid <slave> ... <option> ...] did not accept x or
    # ^ as the first slave argument until Tk 4.2.  Curiously,
    # [grid configure ...] still doesn't in Tk 8.0.5 or Tk 8.1b1 .
    if {[llength [info commands grid]]} {
        rename grid Tk4.1_grid
        array set _Tk4.1_grid {"" ""}
        unset _Tk4.1_grid()

        proc Tk4.1_gridParseSlaves {args} {
            set slaves {}
            set firstOption 0
            foreach arg $args {
                if {[string match .* $arg]} {
                    incr firstOption
                    lappend slaves $arg
                    continue
                }
                switch -exact -- $arg {
                    - -
                    ^ -
                    x {
                        incr firstOption
                        lappend slaves $arg
                        continue
                    }
                    default {
                        break
                    }
                }
            }
            set options [lrange $args $firstOption end]
            return [list $slaves $options]
        }

        proc grid {option args} {
            global _Tk4.1_grid
            set myName [lindex [info level 0] 0]
            if {[string match .* $option]} {
                # Supply optional subcommand 'configure' and re-call
                return [uplevel [list $myName configure $option] $args]
            }
            if {[llength $args] == 0} {
                return -code error "wrong # args:\
                        should be \"$myName option arg ?arg ...?\""
            }
            if {[string match {[x^]} $option]} {
                foreach {slvs opts} [eval Tk4.1_gridParseSlaves $option $args] {}
                if {[llength $opts] % 2} {
                    return -code error "extra option \"[lindex $opts end]\"\
                            (option with no value?)"
                }
                array set optArr $opts
                set numSlaves [llength $slvs]
                set defCol 0
                for {set i 0} {$i < $numSlaves} {incr i} {
                    set slv [lindex $slvs $i]
                    if {[string match {[x^]} $slv]} {
                        incr defCol
                        if {[string match ^ $slv]} {
                            # Fill in code here to expand the slave in
                            # the row above, if it ever turns out we need
                            # that feature.
                        }
                    } elseif {[string match .* $slv]} {
                        # Found a real window
                        if {[info exists optArr(-column)]} {
                            # Column explicitly given; relative placement
                            # stuff has no effect.  Call grid configure
                            # with first real window moved up to front
                            return [uplevel Tk4.1_grid configure \
                                    [lrange $slvs $i end] $opts
                        }
                        # No column explicitly given; honor ^ and x
                        # First collect all trailing -
                        set rs [list $slv]
                        incr i
                        while {[string match - [lindex $slvs $i]]} {
                            lappend rs -
                            incr i
                        }
                        incr i -1
                        if {![info exists optArr(-row)]} {
                            # grid the real window and its trailing -
                            uplevel Tk4.1_grid configure $rs \
                                    [array get optArr] -column $defCol
                            # Capture its row to apply to other slaves
                            array set foo [Tk4.1_grid info $slv]
                            set optArr(-row) $foo(-row)
                        } else {
                            # grid the real window and its trailing -
                            uplevel Tk4.1_grid configure $rs \
                                    [array get optArr] -column $defCol
                        }
                        incr defCol [llength $rs]
                    } else {
                        return -code error "Must specify window\
                                before shortcut '-'."
                    }
                }
                return
            }
            switch -exact -- $option {
                b    -
                bb   -
                bbo  -
                bbox {
                    switch -exact -- [llength $args] {
                        1 {
                           return -code error "$myName bbox syntax not\
                                   supported; Upgrade to Tk 4.2 or higher"
                        }
                        5 {
                           return -code error "$myName bbox syntax not\
                                   supported; Upgrade to Tk 4.2 or higher"
                        }
                        default {
                            uplevel Tk4.1_grid bbox $args
                        }
                    }
                }
                con       -
                conf      -
                confi     -
                config    -
                configu   -
                configur  -
                configure {
                    foreach {slaves opts} [eval Tk4.1_gridParseSlaves $args] {}
                    # First restore saved options from prior 'remove' if any
                    foreach slave $slaves {
                        if {[info exists _Tk4.1_grid($slave)]} { 
                            uplevel [list Tk4.1_grid configure $slave] \
                                    [set _Tk4.1_grid($slave)]
                            unset _Tk4.1_grid($slave)
                        }
                    }
                    uplevel Tk4.1_grid configure $slaves $opts
                }
                f      -
                fo     -
                for    -
                forg   -
                forge  -
                forget {
                    foreach slave $args {
                        catch {unset _Tk4.1_grid($slave)}
                    }
                    uplevel Tk4.1_grid forget $args
                }
                re     -
                rem    -
                remo   -
                remov  -
                remove {
                    foreach slave $args {
                        set slaveConf [Tk4.1_grid info $slave]
                        if {[llength $slaveConf]} {
                            array set _Tk4.1_grid [list $slave $slaveConf]
                        }
                        uplevel Tk4.1_grid forget $slave
                    }
                }
                ro              -
                row             -
                rowc            -
                rowco           -
                rowcon          -
                rowconf         -
                rowconfi        -
                rowconfig       -
                rowconfigu      -
                rowconfigur     -
                rowconfigure    -
                col             -
                colu            -
                colum           -
                column          -
                columnc         -
                columnco        -
                columncon       -
                columnconf      -
                columnconfi     -
                columnconfig    -
                columnconfigu   -
                columnconfigur  -
                columnconfigure {
                    if {[llength $args] < 2} {
                        return -code error "wrong # args:\
                                should be \"$myName $option master index\
                                ?-option value...?\""
                    }
                    set master [lindex $args 0]
                    set index [lindex $args 1]
                    set options [lrange $args 2 end]
                    if {[llength $options] == 0} {
                        set retList -minsize
                        lappend retList \
                                [Tk4.1_grid $option $master $index -minsize]
                        lappend retList -pad 0 -weight
                        lappend retList \
                                [Tk4.1_grid $option $master $index -weight]
                        return $retList
                    }
                    if {[llength $options] == 1} {
                        if {[string match -pad [lindex $options 0]]} {
                            return 0
                        } else {
                            uplevel [list Tk4.1_grid $option $master \
                                    $index] $options
                        }
                    } elseif {[llength $options] % 2} {
                        return -code error "wrong # args:\
                                should be \"$myName $option master index\
                                ?-option value...?\""
                    } else {
                        set passOpts {}
                        foreach {opt val} $options {
                            if {[string compare -pad $opt]} {
                                lappend passOpts $opt $val
                            }
                        }       
                        if {[llength $passOpts]} {
                            uplevel [list Tk4.1_grid $option $master $index] \
                                    $passOpts
                        }
                    }
                }
                default {
                    uplevel Tk4.1_grid $option $args
                }
            }
        }
    }

} else {
    Oc_Log Log "OOMMF software does not support 
Tcl version 7.5 on non-Unix platforms.  
Please upgrade your Tcl installation." panic 
    exit
}
    
# Continue with bug patches for the next Tcl version
source [file join [file dirname [info script]] bug76.tcl]

